@@ -4,7 +4,7 @@ import json
 import itertools
 from collections import OrderedDict, defaultdict
 from functools import partial, reduce
-from typing import Optional, Dict, List, Any
+from typing import Optional, Dict, List, Any, Union, Mapping
 
 import numpy as np
 from marshmallow import ValidationError as MarshmallowValidationError
@@ -13,12 +13,14 @@ from paramtools import utils
 from paramtools.schema import ParamToolsSchema
 from paramtools.schema_factory import SchemaFactory
 from paramtools.select import (
+    select,
     select_eq,
-    select_gt_ix,
+    select_ne,
     select_gt,
     select_gte,
     select_lt,
     select_lte,
+    make_cmp_func,
 )
 from paramtools.tree import Tree
 from paramtools.typing import ValueObject
@@ -42,10 +44,8 @@ class Parameters:
     def __init__(
         self,
         initial_state: Optional[dict] = None,
-        array_first: Optional[bool] = False,
-        label_to_extend: str = None,
-        uses_extend_func: bool = False,
         index_rates: Optional[dict] = None,
+        **ops,
     ):
         schemafactory = SchemaFactory(self.defaults)
         (
@@ -65,7 +65,7 @@ class Parameters:
         self._validator_schema.context["spec"] = self
         self._warnings = {}
         self._errors = {}
-        self._state = initial_state or {}
+        self._state = self.parse_labels(**(initial_state or {}))
         self._search_trees = {}
         self.index_rates = index_rates or self.index_rates
 
@@ -74,18 +74,21 @@ class Parameters:
         # class attribute: middle importance
         # schema action: least important
         # default value if three above are not specified.
-        ops = [
-            ("array_first", array_first, False),
-            ("label_to_extend", label_to_extend, None),
-            ("uses_extend_func", uses_extend_func, False),
+        default_ops = [
+            ("array_first", False),
+            ("label_to_extend", None),
+            ("uses_extend_func", False),
         ]
         schema_ops = self._schema.get("operators", {})
-        for name, init_value, default in ops:
-            user_vals = [init_value, getattr(self, name), schema_ops.get(name)]
-            for value in user_vals:
-                if value != default and value is not None:
-                    setattr(self, name, value)
-                    break
+        for name, default in default_ops:
+            if name in ops:
+                setattr(self, name, ops.get(name))
+            elif getattr(self, name, None) != default:
+                setattr(self, name, getattr(self, name))
+            elif name in schema_ops:
+                setattr(self, name, schema_ops[name])
+            else:
+                setattr(self, name, default)
 
         if self.label_to_extend:
             prev_array_first = self.array_first
@@ -104,20 +107,24 @@ class Parameters:
 
     def set_state(self, **labels):
         """
-        Sets state for the Parameters instance. The state, label_grid, and
+        Sets state for the Parameters instance. The `_state`, `label_grid`, and
         parameter attributes are all updated with the new state.
 
-        Raises:
-            ValidationError if the labels kwargs contain labels that are not
-                specified in schema.json or if the label values fail the
-                validator set for the corresponding label in schema.json.
+        Use the `view_state` method to inspect the current state of the instance,
+        and use the `clear_state` method to revert to the default state.
+
+        **Raises**
+
+          - `ValidationError` if the labels kwargs contain labels that are not
+            specified in schema.json or if the label values fail the
+            validator set for the corresponding label in schema.json.
         """
 
         self._set_state(**labels)
 
     def clear_state(self):
         """
-        Reset the state of the Parameters instance.
+        Reset the state of the `Parameters` instance.
         """
         self._state = {}
         self.label_grid = copy.deepcopy(self._stateless_label_grid)
@@ -142,33 +149,57 @@ class Parameters:
 
     def adjust(
         self,
-        params_or_path,
-        ignore_warnings=False,
-        raise_errors=True,
-        extend_adj=True,
+        params_or_path: Union[str, Mapping[str, List[ValueObject]]],
+        ignore_warnings: bool = False,
+        raise_errors: bool = True,
+        extend_adj: bool = True,
+        clobber: bool = True,
     ):
         """
         Deserialize and validate parameter adjustments. `params_or_path`
         can be a file path or a `dict` that has not been fully deserialized.
         The adjusted values replace the current values stored in the
-        corresponding parameter attributes. This simply calls a private
-        method `_adjust` to do the upate. Creating this layer on top of
-        `_adjust` makes it easy to subclass Parameters and implement custom
-        `adjust` methods.
+        corresponding parameter attributes.
 
-        Returns: parsed, validated parameters.
+        If `clobber` is `True` and extend mode is on, then all future values
+        for a given parameter be replaced by the values in the adjustment.
+        If `clobber` is `False` and extend mode is on, then user-defined values
+        will not be replaced by values in this adjustment. Only values that
+        were added automatically via the extend method will be updated.
 
-        Raises:
-            marshmallow.exceptions.ValidationError if data is not valid.
+        This simply calls a private method `_adjust` to do the upate. Creating
+        this layer on top of `_adjust` makes it easy to subclass `Parameters` and
+        implement custom `adjust` methods.
 
-            ParameterUpdateException if label values do not match at
-                least one existing value item's corresponding label values.
+        **Parameters**
+
+          - `params_or_path`: Adjustment that is either a `dict`, file path, or
+            JSON string.
+          - `ignore_warnings`: Whether to raise an error on warnings or ignore them.
+          - `raise_errors`: Either raise errors or simply store the error messages.
+          - `extend_adj`: If in extend mode, this is a flag indicating whether to
+            extend the adjustment values or not.
+          - `clobber`: If in extend mode, this is a flag indicating whether to
+            override all values, including user-defined values, or to only
+            override automatically created values.
+
+        **Returns**
+
+          - `params`: Parsed, validated parameters.
+
+        **Raises**
+
+          - `marshmallow.exceptions.ValidationError` if data is not valid.
+
+          - `ParameterUpdateException` if label values do not match at
+            least one existing value item's corresponding label values.
         """
         return self._adjust(
             params_or_path,
             ignore_warnings=ignore_warnings,
             raise_errors=raise_errors,
             extend_adj=extend_adj,
+            clobber=clobber,
         )
 
     def _adjust(
@@ -178,13 +209,20 @@ class Parameters:
         raise_errors=True,
         extend_adj=True,
         is_deserialized=False,
+        clobber=True,
     ):
         """
         Internal method for performing adjustments.
         """
         # Validate user adjustments.
         if is_deserialized:
-            parsed_params = params_or_path
+            parsed_params = {}
+            try:
+                parsed_params = self._validator_schema.load(
+                    params_or_path, ignore_warnings, is_deserialized=True
+                )
+            except MarshmallowValidationError as ve:
+                self._parse_validation_messages(ve.messages, params_or_path)
         else:
             params = self.read_params(params_or_path)
             parsed_params = {}
@@ -200,32 +238,46 @@ class Parameters:
                 extend_grid = self._stateless_label_grid[self.label_to_extend]
                 to_delete = defaultdict(list)
                 backup = {}
+                cmp_funcs = self.label_validators[
+                    self.label_to_extend
+                ].cmp_funcs()
+                gt_cmp_func = make_cmp_func(cmp_funcs["gt"], all_or_any=all)
                 for param, vos in parsed_params.items():
                     for vo in utils.grid_sort(
                         vos, self.label_to_extend, extend_grid
                     ):
+
                         if self.label_to_extend in vo:
-                            gt = select_gt_ix(
-                                self._data[param]["value"],
-                                True,
-                                {
-                                    self.label_to_extend: vo[
-                                        self.label_to_extend
-                                    ]
-                                },
-                                extend_grid,
-                                tree=self._search_trees.get(param),
+                            query_args = {
+                                self.label_to_extend: vo[self.label_to_extend]
+                            }
+                            if clobber:
+                                queryset = self._data[param]["value"]
+                                tree = self._search_trees.get(param)
+                            else:
+                                queryset = self.select_eq(
+                                    param, strict=True, _auto=True
+                                )
+                                tree = None
+                            gt = select(
+                                queryset,
+                                strict=False,
+                                cmp_func=gt_cmp_func,
+                                labels=query_args,
+                                tree=tree,
                             )
-                            eq = select_eq(
+                            to_delete[param] += select_eq(
                                 gt,
-                                True,
-                                utils.filter_labels(
-                                    vo, drop=[self.label_to_extend, "value"]
+                                strict=False,
+                                labels=utils.filter_labels(
+                                    vo,
+                                    drop=[
+                                        self.label_to_extend,
+                                        "value",
+                                        "_auto",
+                                    ],
                                 ),
                             )
-                            to_delete[param] += [
-                                dict(td, **{"value": None}) for td in eq
-                            ]
                     # make copy of value objects since they
                     # are about to be modified
                     backup[param] = copy.deepcopy(self._data[param]["value"])
@@ -234,7 +286,7 @@ class Parameters:
                     self.array_first = False
 
                     # delete params that will be overwritten out by extend.
-                    self._adjust(
+                    self.delete(
                         to_delete,
                         extend_adj=False,
                         raise_errors=True,
@@ -326,14 +378,25 @@ class Parameters:
         to_delete = {}
         for param, vos in parsed_params.items():
             to_delete[param] = [dict(vo, **{"value": None}) for vo in vos]
+            self._update_param(param, to_delete[param])
 
-        return self._adjust(
-            to_delete,
-            ignore_warnings=ignore_warnings,
-            raise_errors=raise_errors,
-            extend_adj=extend_adj,
-            is_deserialized=True,
-        )
+        if self.label_to_extend is not None and extend_adj:
+            self.extend()
+
+        self._validator_schema.context["spec"] = self
+
+        has_errors = bool(self._errors.get("messages"))
+        has_warnings = bool(self._warnings.get("messages"))
+        # throw error if raise_errors is True or ignore_warnings is False
+        if (raise_errors and has_errors) or (
+            not ignore_warnings and has_warnings
+        ):
+            raise self.validation_error
+
+        # Update attrs for params that were adjusted.
+        self._set_state(params=to_delete.keys())
+
+        return to_delete
 
     @property
     def errors(self):
@@ -391,26 +454,29 @@ class Parameters:
 
     def specification(
         self,
-        use_state=True,
-        meta_data=False,
-        include_empty=False,
-        serializable=False,
-        sort_values=False,
+        use_state: bool = True,
+        meta_data: bool = False,
+        include_empty: bool = False,
+        serializable: bool = False,
+        sort_values: bool = False,
         **labels,
     ):
         """
         Query value(s) of all parameters along labels specified in
         `labels`.
 
-        Parameters:
-            - use_state: If true, use the instance's state for the select operation.
-            - meta_data: If true, include information like the parameter
-                description and title.
-            - include_empty: If true, include parameters that do not meet the label query.
-            - serializable: If true, return data that is compatible with `json.dumps`.
+        **Parameters**
 
-        Returns: serialized data of shape
-            {"param_name": [{"value": val, "label0": ..., }], ...}
+          - `use_state`: Use the instance's state for the select operation.
+          - `meta_data`: Include information like the parameter
+            `description` and title.
+          - `include_empty`: Include parameters that do not meet the label query.
+          - `serializable`: Return data that is compatible with `json.dumps`.
+          - `sort_values`: Sort values by the `label` order.
+
+        **Returns**
+
+          - `dict` of parameter names and data.
         """
         if use_state:
             labels.update(self._state)
@@ -419,7 +485,9 @@ class Parameters:
         for param in self._validator_schema.fields:
             result = self.select_eq(param, False, **labels)
             if sort_values and result:
-                self.sort_values(data={param: result}, has_meta_data=False)
+                result = self.sort_values(
+                    data={param: result}, has_meta_data=False
+                )[param]
             if result or include_empty:
                 if meta_data:
                     param_data = self._data[param]
@@ -437,27 +505,43 @@ class Parameters:
         else:
             return all_params
 
-    def to_array(self, param):
+    def to_array(self, param, **labels):
         """
         Convert a Value object to an n-labelal array. The list of Value
         objects must span the specified parameter space. The parameter space
         is defined by inspecting the label validators in schema.json
         and the state attribute of the Parameters instance.
 
-        Returns: n-labelal NumPy array.
+        **Parameters**
+          - `param`: Name of parameter that will be used to create array.
+          - `labels`: Optionally, override instance state.
 
-        Raises:
-            InconsistentLabelsException: Value objects do not have consistent
-                labels.
-            SparseValueObjectsException: Value object does not span the
-                entire space specified by the Order object.
-            ParamToolsError: Parameter is an array type and has labels.
-                This is not supported by ParamTools when using array_first.
+        **Returns**
+
+          - `arr`: NumPy array created from list of value objects.
+
+        **Raises**
+
+          - `InconsistentLabelsException`: Value objects do not have consistent
+            labels.
+          - `SparseValueObjectsException`: Value object does not span the
+            entire space specified by the Order object.
+          - `ParamToolsError`: Parameter is an array type and has labels.
+            This is not supported by ParamTools when using array_first.
         """
-        value_items = self.select_eq(param, False, **self._state)
+        label_grid = copy.deepcopy(self.label_grid)
+        state = copy.deepcopy(self._state)
+        if labels:
+            parsed_labels = self.parse_labels(**labels)
+            label_grid.update(parsed_labels)
+            state.update(parsed_labels)
+        value_items = self.select_eq(param, False, **state)
         if not value_items:
             return np.array([])
-        label_order, value_order = self._resolve_order(param, value_items)
+
+        label_order, value_order = self._resolve_order(
+            param, value_items, label_grid
+        )
         shape = []
         for label in label_order:
             shape.append(len(value_order[label]))
@@ -486,7 +570,8 @@ class Parameters:
             else:
                 return data_type(value)
         exp_full_shape = reduce(lambda x, y: x * y, shape)
-        if len(value_items) != exp_full_shape:
+        act_full_shape = len(value_items)
+        if act_full_shape != exp_full_shape:
             # maintains label value order over value objects.
             exp_grid = list(itertools.product(*value_order.values()))
             # preserve label value order for each value object by
@@ -520,16 +605,27 @@ class Parameters:
             arr[ix] = vi["value"]
         return arr
 
-    def from_array(self, param, array=None):
+    def from_array(self, param, array=None, **labels):
         """
         Convert NumPy array to a Value object.
 
-        Returns:
-            Value object (shape: [{"value": val, labels:...}])
+        **Parameters**
 
-        Raises:
-            InconsistentLabelsException: Value objects do not have consistent
-                labels.
+          - `param`: Name of parameter to convert to a list of value objects.
+          - `array`: Optionally, provide a NumPy array to convert into a list
+            of value objects. If not specified, the value at `self.param` will
+            be used.
+          - `labels`: Optionally, override instance state.
+
+
+        **Returns**
+
+          - List of `ValueObjects`
+
+        **Raises**
+
+          - `InconsistentLabelsException`: Value objects do not have consistent
+            labels.
         """
         if array is None:
             array = getattr(self, param)
@@ -538,8 +634,18 @@ class Parameters:
                     "A NumPy Ndarray should be passed to this method "
                     "or the instance attribute should be an array."
                 )
-        value_items = self.select_eq(param, False, **self._state)
-        label_order, value_order = self._resolve_order(param, value_items)
+
+        label_grid = copy.deepcopy(self.label_grid)
+        state = copy.deepcopy(self._state)
+        if labels:
+            parsed_labels = self.parse_labels(**labels)
+            label_grid.update(parsed_labels)
+            state.update(parsed_labels)
+
+        value_items = self.select_eq(param, False, **state)
+        label_order, value_order = self._resolve_order(
+            param, value_items, label_grid
+        )
         label_values = itertools.product(*value_order.values())
         label_indices = itertools.product(
             *map(lambda x: range(len(x)), value_order.values())
@@ -553,21 +659,34 @@ class Parameters:
 
     def extend(
         self,
-        label_to_extend=None,
-        label_to_extend_values=None,
-        params=None,
-        raise_errors=True,
-        ignore_warnings=False,
+        label: Optional[str] = None,
+        label_values: Optional[List[Any]] = None,
+        params: Optional[List[str]] = None,
+        raise_errors: bool = True,
+        ignore_warnings: bool = False,
     ):
         """
-        Extend parameters along label_to_extend.
+        Extend parameters along `label`.
 
-        Raises:
-            InconsistentLabelsException: Value objects do not have consistent
-                labels.
+        **Parameters**
+
+        - `label`: Label to extend values along. By default, `label_to_extend`
+          is used.
+        - `label_values`: values of `label` to extend. By default, this is a grid
+          created from the valid values of `label_to_extend`.
+        - `params`: Parameters to extend. By default, all parameters are extended.
+        - `raise_errors`: Whether `adjust` should raise or store errors.
+        - `ignore_warnings`: Whether `adjust` should raise or ignore warnings.
+
+        **Raises**
+
+          - `InconsistentLabelsException`: Value objects do not have consistent
+            labels.
         """
-        if label_to_extend is None:
-            label_to_extend = self.label_to_extend
+        if label is None:
+            label = self.label_to_extend
+        else:
+            label = label
 
         spec = self.specification(meta_data=True)
         if params is not None:
@@ -576,86 +695,87 @@ class Parameters:
                 for param, data in spec.items()
                 if param in params
             }
-        extend_grid = (
-            label_to_extend_values
-            or self._stateless_label_grid[label_to_extend]
-        )
+        full_extend_grid = self._stateless_label_grid[label]
+        if label_values is not None:
+            labels = self.parse_labels(**{label: label_values})
+            extend_grid = labels[label]
+        else:
+            extend_grid = self._stateless_label_grid[label]
+
+        cmp_funcs = self.label_validators[label].cmp_funcs(choices=extend_grid)
+        gt_cmp_func = make_cmp_func(cmp_funcs["gt"], all_or_any=all)
+
         adjustment = defaultdict(list)
         for param, data in spec.items():
-            if not any(label_to_extend in vo for vo in data["value"]):
+            if not any(label in vo for vo in data["value"]):
                 continue
             extended_vos = set()
             for vo in sorted(
-                data["value"],
-                key=lambda val: extend_grid.index(val[label_to_extend]),
+                data["value"], key=lambda val: cmp_funcs["key"](val[label])
             ):
                 hashable_vo = utils.hashable_value_object(vo)
                 if hashable_vo in extended_vos:
                     continue
                 else:
                     extended_vos.add(hashable_vo)
-                gt = select_gt_ix(
+                gt = select(
                     self._data[param]["value"],
                     False,
-                    {label_to_extend: vo[label_to_extend]},
-                    extend_grid,
+                    gt_cmp_func,
+                    {label: vo[label]},
                     tree=self._search_trees.get(param),
                 )
                 eq = select_eq(
                     gt,
                     False,
-                    utils.filter_labels(vo, drop=["value", label_to_extend]),
+                    utils.filter_labels(vo, drop=["value", label, "_auto"]),
                 )
                 extended_vos.update(map(utils.hashable_value_object, eq))
                 eq += [vo]
 
-                defined_vals = {eq_vo[label_to_extend] for eq_vo in eq}
+                defined_vals = {eq_vo[label] for eq_vo in eq}
 
                 missing_vals = sorted(
-                    set(extend_grid) - defined_vals,
-                    key=lambda val: extend_grid.index(val),
+                    set(extend_grid) - defined_vals, key=cmp_funcs["key"]
                 )
 
                 if not missing_vals:
                     continue
 
                 extended = defaultdict(list)
+                for vo in eq:
+                    extended[vo[label]].append(vo)
+
+                skl = utils.SortedKeyList(extended.keys(), cmp_funcs["key"])
 
                 for val in missing_vals:
-                    eg_ix = extend_grid.index(val)
-                    if eg_ix == 0:
-                        first_defined_value = min(
-                            defined_vals,
-                            key=lambda val: extend_grid.index(val),
-                        )
-                        value_objects = select_eq(
-                            eq, True, {label_to_extend: first_defined_value}
-                        )
-                    elif extend_grid[eg_ix - 1] in extended:
-                        value_objects = extended.pop(extend_grid[eg_ix - 1])
+                    lte_val = skl.lte(val)
+                    if lte_val is not None:
+                        closest_val = lte_val
+                    else:
+                        closest_val = skl.gte(val)
+
+                    if closest_val in extended:
+                        value_objects = extended.pop(closest_val)
                     else:
-                        prev_defined_value = extend_grid[eg_ix - 1]
                         value_objects = select_eq(
-                            eq, True, {label_to_extend: prev_defined_value}
+                            eq, False, {label: closest_val}
                         )
                     # In practice, value_objects has length one.
                     # Theoretically, there could be multiple if the inital value
                     # object had less labels than later value objects and thus
                     # matched multiple value objects.
                     for value_object in value_objects:
-                        ext = dict(value_object, **{label_to_extend: val})
+                        ext = dict(value_object, **{label: val})
                         ext = self.extend_func(
-                            param,
-                            ext,
-                            value_object,
-                            extend_grid,
-                            label_to_extend,
+                            param, ext, value_object, full_extend_grid, label
                         )
                         extended_vos.add(
                             utils.hashable_value_object(value_object)
                         )
                         extended[val].append(ext)
-                        adjustment[param].append(ext)
+                        skl.insert(val)
+                        adjustment[param].append(dict(ext, _auto=True))
         # Ensure that the adjust method of paramtools.Parameter is used
         # in case the child class also implements adjust.
         self._adjust(
@@ -663,6 +783,7 @@ class Parameters:
             extend_adj=False,
             ignore_warnings=ignore_warnings,
             raise_errors=raise_errors,
+            is_deserialized=True,
         )
 
     def extend_func(
@@ -671,35 +792,38 @@ class Parameters:
         extend_vo: ValueObject,
         known_vo: ValueObject,
         extend_grid: List,
-        label_to_extend: str,
+        label: str,
     ):
         """
         Function for applying indexing rates to parameter values as they
-        are extended. Projects may implement their own extend_func by
-        overriding this one. Projects need to write their own indexing_rate
+        are extended. Projects may implement their own `extend_func` by
+        overriding this one. Projects need to write their own `indexing_rate`
         method for returning the correct indexing rate for a given parameter
-        and value of label_to_extend (abbreviated to lte_val).
+        and value of `label`.
 
-        returns: extended_vo
+        **Returns**
+
+          - `extend_vo`: New `ValueObject`.
         """
         if not self.uses_extend_func or not self._data[param].get(
             "indexed", False
         ):
             return extend_vo
 
-        known_val = known_vo[label_to_extend]
+        known_val = known_vo[label]
         known_ix = extend_grid.index(known_val)
 
-        toext_val = extend_vo[label_to_extend]
+        toext_val = extend_vo[label]
         toext_ix = extend_grid.index(toext_val)
 
         if toext_ix > known_ix:
             # grow value according to the index rate supplied by the user defined
             # self.indexing_rate method.
-            v = extend_vo["value"] * (
-                1 + self.get_index_rate(param, known_val)
-            )
-            extend_vo["value"] = np.round(v, 2) if v < 9e99 else 9e99
+            for ix in range(known_ix, toext_ix):
+                v = extend_vo["value"] * (
+                    1 + self.get_index_rate(param, extend_grid[ix])
+                )
+                extend_vo["value"] = np.round(v, 2) if v < 9e99 else 9e99
         else:
             # shrink value according to the index rate supplied by the user defined
             # self.indexing_rate method.
@@ -720,30 +844,46 @@ class Parameters:
         """
         return self.index_rates[lte_val]
 
-    def _set_state(self, params=None, **labels):
+    def parse_labels(self, **labels):
         """
-        Private method for setting the state on a Parameters instance. Internal
-        methods can set which params will be updated. This is helpful when a set
-        of parameters are adjusted and only their attributes need to be updated.
+        Parse and validate labels.
+
+        **Returns**
+
+        - Parsed and validated labels.
         """
+        parsed = defaultdict(list)
         messages = {}
         for name, values in labels.items():
             if name not in self.label_validators:
                 messages[name] = f"{name} is not a valid label."
                 continue
             if not isinstance(values, list):
-                values = [values]
-            for value in values:
+                list_values = [values]
+            else:
+                list_values = values
+            assert isinstance(list_values, list)
+            for value in list_values:
                 try:
-                    self.label_validators[name].deserialize(value)
+                    parsed[name].append(
+                        self.label_validators[name].deserialize(value)
+                    )
                 except MarshmallowValidationError as ve:
                     messages[name] = str(ve)
         if messages:
             raise ValidationError({"errors": messages}, labels=None)
+        return parsed
+
+    def _set_state(self, params=None, **labels):
+        """
+        Private method for setting the state on a Parameters instance. Internal
+        methods can set which params will be updated. This is helpful when a set
+        of parameters are adjusted and only their attributes need to be updated.
+        """
+        labels = self.parse_labels(**labels)
         self._state.update(labels)
         for label_name, label_value in self._state.items():
-            if not isinstance(label_value, list):
-                label_value = [label_value]
+            assert isinstance(label_value, list)
             self.label_grid[label_name] = label_value
         spec = self.specification(include_empty=True, **self._state)
         if params is not None:
@@ -758,23 +898,24 @@ class Parameters:
             else:
                 setattr(self, name, value)
 
-    def _resolve_order(self, param, value_items):
+    def _resolve_order(self, param, value_items, label_grid):
         """
         Resolve the order of the labels and their values by
         inspecting data in the label grid values.
 
-        The label grid for all labels is stored in the label_grid
-        attribute. The labels to be used are the ones that are specified
+        The labels to be used are the ones that are specified
         for each value object. Note that the labels must be specified
         _consistently_ for all value objects, i.e. none can be added or omitted
         for any value object in the list.
 
-        Returns:
-            label_order: The label order.
-            value_order: The values, in order, for each label.
+        **Returns**
 
-        Raises:
-            InconsistentLabelsException: Value objects do not have consistent
+            - `label_order`: The label order.
+            - `value_order`: The values, in order, for each label.
+
+        **Raises**
+
+            - `InconsistentLabelsException`: Value objects do not have consistent
                 labels.
         """
         used = utils.consistent_labels(value_items)
@@ -783,7 +924,7 @@ class Parameters:
                 f"were added or omitted for some value object(s)."
             )
         label_order, value_order = [], {}
-        for label_name, label_values in self.label_grid.items():
+        for label_name, label_values in label_grid.items():
             if label_name in used:
                 label_order.append(label_name)
                 value_order[label_name] = label_values
@@ -797,42 +938,50 @@ class Parameters:
             self._validator_schema.fields[param].schema.fields["value"].np_type
         )
 
-    def select_eq(self, param, exact_match=True, **labels):
+    def select_eq(self, param, strict=True, **labels):
         return select_eq(
             self._data[param]["value"],
-            exact_match,
+            strict,
             labels,
             tree=self._search_trees.get(param),
         )
 
-    def select_gt(self, param, exact_match=True, **labels):
+    def select_ne(self, param, strict=True, **labels):
+        return select_ne(
+            self._data[param]["value"],
+            strict,
+            labels,
+            tree=self._search_trees.get(param),
+        )
+
+    def select_gt(self, param, strict=True, **labels):
         return select_gt(
             self._data[param]["value"],
-            exact_match,
+            strict,
             labels,
             tree=self._search_trees.get(param),
         )
 
-    def select_gte(self, param, exact_match=True, **labels):
+    def select_gte(self, param, strict=True, **labels):
         return select_gte(
             self._data[param]["value"],
-            exact_match,
+            strict,
             labels,
             tree=self._search_trees.get(param),
         )
 
-    def select_lt(self, param, exact_match=True, **labels):
+    def select_lt(self, param, strict=True, **labels):
         return select_lt(
             self._data[param]["value"],
-            exact_match,
+            strict,
             labels,
             tree=self._search_trees.get(param),
         )
 
-    def select_lte(self, param, exact_match=True, **labels):
+    def select_lte(self, param, strict=True, **labels):
         return select_lte(
             self._data[param]["value"],
-            exact_match,
+            strict,
             labels,
             tree=self._search_trees.get(param),
         )
@@ -982,15 +1131,21 @@ class Parameters:
 
     def sort_values(self, data=None, has_meta_data=True):
         """
-        Sort value objects for all parameters in data according
-        to the order specified in schema. If data is not specified,
-        the existing value objects are used. User specified data
-        can be:
-            {param: [...value objects]}
+        Sort value objects for all parameters in `data` according
+        to the order specified in `schema`.
 
-            or
 
-            {param: {"value": [...value objects]}}
+        **Parameters**
+
+          - `data`: Parameter data to be sorted. This should be a
+            `dict` of parameter names and values. If `data` is `None`,
+            the current values will be sorted.
+          - `has_meta_data`: Whether parameter values should be accessed
+            directly or through the "value" attribute.
+
+        **Returns**
+
+          - Sorted data.
         """
 
         def keyfunc(vo, label, label_values):
@@ -999,15 +1154,6 @@ class Parameters:
             else:
                 return -1
 
-        # nothing to do if labels aren't specified
-        if not self._stateless_label_grid:
-            return
-
-        # iterate over labels so that the first label's order
-        # takes precedence.
-        label_grid = self._stateless_label_grid
-        order = list(reversed(label_grid))
-
         if data is None:
             data = self._data
             update_attrs = True
@@ -1018,18 +1164,36 @@ class Parameters:
         else:
             update_attrs = False
 
-        for param, param_data in data.items():
+        # nothing to do if labels aren't specified
+        if not self._stateless_label_grid:
+            return data
+
+        # iterate over labels so that the first label's order
+        # takes precedence.
+        label_grid = self._stateless_label_grid
+        order = list(reversed(label_grid))
+
+        for param in data:
             for label in order:
                 label_values = label_grid[label]
                 pfunc = partial(
                     keyfunc, label=label, label_values=label_values
                 )
                 if has_meta_data:
-                    param_data["value"].sort(key=pfunc)
+                    data[param]["value"] = sorted(
+                        data[param]["value"], key=pfunc
+                    )
                 else:
-                    param_data.sort(key=pfunc)
+                    data[param] = sorted(data[param], key=pfunc)
 
             # Only update attributes when array first is off, since
             # value order will not affect how arrays are constructed.
             if update_attrs and has_meta_data and not self.array_first:
-                setattr(self, param, param_data["value"])
+                attr_vals = select_eq(
+                    data[param]["value"], strict=True, labels=self._state
+                )
+                sorted_values = self.sort_values(
+                    {param: attr_vals}, has_meta_data=False
+                )[param]
+                setattr(self, param, sorted_values)
+        return data
